# R code description
#'#' in front of a sentence means a comment. It does not affect an R code result.
# It is recommended that this file be opened with an R syntax highlighting editor such as 'Notepad ++', 'Atom', 'Visual Studio Code', and 'RStudio'.
# Notepad++ : https://notepad-plus-plus.org
# Atom : https://atom.io/
# Visual Studio Code : https://code.visualstudio.com/
# RStudio : https://www.rstudio.com/


###########################################
## Whole genome visualization (Figure 1) ##
###########################################

#===========================================
# gff file reading
#===========================================
# This gff file was downloaded after RAST annotation 
read.gff<-read.csv("path/to/TFH56.gff",header=F, sep = "\t")[-1,]
colnames(read.gff) <- c("Contig", "FIG", "Feature", "start", "end", "x", "strand", "frame", "identity")

head(read.gff)

#===========================
# Input of contig length
#===========================
C1L <- c(1, 7488586)
C2L <- c(1 + C1L[2], 127820 + C1L[2])
C3L <- c(1 + C2L[2], 50115 + C2L[2])

Contigs <- as.data.frame(t(data.frame(C1L,C2L,C3L)))
colnames(Contigs) <-c("start","end")

#===========================
# Gathering length information with gff file
#===========================
levels(read.gff$Contig)
contig1 <- subset(read.gff, Contig=="tig00000001|arrow")
contig2 <- subset(read.gff, Contig=="tig00000011|arrow")
contig3 <- subset(read.gff, Contig=="tig00000013|arrow")

contig2[,4] <- contig2[,4]+C1L[2]
contig2[,5] <- contig2[,5]+C1L[2]

contig3[,4] <- contig3[,4]+C2L[2]
contig3[,5] <- contig3[,5]+C2L[2]

Genomeinfo <- rbind(contig1,contig2,contig3)

# Split "+" and "-" direction
Genomeinfo$strand2 <- ifelse(Genomeinfo$strand=="+",1,0)

#==========================================#
# reading whole genome fasta file
#===========================================

WGfasta <- read.csv("~/TFH56.fna",header = F)
WGS <- paste(WGfasta[-grep("^>", WGfasta$V1),], collapse = "")
nchar(WGS)

# Divide by 10000bp 
WGSdivide <- function(data){
  a <- as.integer(nchar(data)/10000)+1
  splited <- c()
  for(i in 1:a){
    b <- substr(data,1+(i-1)*10000,i*10000)
    splited <-c(splited,b)
  }
  print(as.data.frame(splited))
}

WGSdivide_result <- WGSdivide(WGS)

#===============
# Calculating GC ratio 
#===============
library(dplyr)
library(stringr)

GCratio <- function(contigseq)
{
  ratio <- c()
  for(i in 1:nrow(contigseq))
  {
    a <- length(gregexpr("[C]|[G]|[c]|[g]", contigseq[i,1])[[1]])
    b <- str_length(contigseq[i,1])
    c <- a/b
    ratio <- c(ratio,c)
  }
  print(ratio)
}

# Calcuating GC ratio per 10000bp
wgsGCratio <- GCratio(WGSdivide_result)
wgsGCratio <- data.frame(wgsGCratio,1:length(wgsGCratio)*10000)
colnames(wgsGCratio) <- c("GCratio", "length")

# Calcuating whole genome GC ratio
gc_ratio_whole <- round(100*GCratio(as.data.frame(WGS)),1)
Contig1 <- substr(WGS, 1, 7488586) 
Contig2 <- substr(WGS, 7488587, 7616406)
Contig3 <- substr(WGS, 7616507, 7666512)
GCratio(as.data.frame(Contig1))
GCratio(as.data.frame(Contig2))
GCratio(as.data.frame(Contig3))

# Max and min of GC ratio
MAX <- round(100*max(wgsGCratio$GCratio),1)
MIN <- round(100*min(wgsGCratio$GCratio),1)

# split above average and below average
GCab <- paste(MAX," ~ ", gc_ratio_whole, "%", sep="")
GCbw<- paste(gc_ratio_whole," ~ ", MIN, "%", sep="")
wgsGCratio$range <- ifelse(wgsGCratio$GCratio>=gc_ratio_whole/100, GCab, GCbw)

#========================
# Drawing whole genome
#========================
library(ggplot2)
head(read.gff)

ylabel <- c("Contigs", "+\nStrand", "-\nStrand", "GC\nratio")
yloc <- c(0.425,0.325,0.16,0)
xloc <- c(7832499.5, 7832499.5, 7832499.5, 7832499.5)
yaxislabel <- data.frame(ylabel,yloc,xloc)

ggplot()+
  geom_rect(data = Contigs, aes(xmin=start,xmax=end, ymin=0.4, ymax=0.45, fill=row.names(Contigs)), color="white",size=0.1)+
  geom_bar(data = wgsGCratio, aes(length, y=(GCratio-gc_ratio_whole/100)*2, fill=range), size=2,stat="identity",inherit.aes = FALSE)+
  geom_segment(data = subset(Genomeinfo, Feature=="CDS"), aes(x=start, xend=end, y=0.15+strand2/6, yend=0.15+strand2/6, color=Feature),stat="identity", size=10)+
  geom_point( data = subset(Genomeinfo, Feature!="CDS"), aes(x=(start+end)/2, y=0.15+strand2/6, color=Feature, shape=Feature), size=2, stat="identity")+
  geom_text(data = yaxislabel, aes(xloc, yloc, label = ylabel), inherit.aes = F, size = 2.8)+
  geom_segment(aes(x=c(0,1:15*500000),xend=c(0,1:15*500000),y=0.47,yend=0.45))+
  ylab("")+
  xlab("")+
  scale_y_continuous(limits = c(-0.5,.47), expand = c(0, 0))+
scale_x_continuous(breaks = c(0,1:15*500000),limits=c(0,8000000),
labels = c("0.0 Mbp","0.5 Mbp","1.0 Mbp","1.5 Mbp","2.0 Mbp","2.5 Mbp","3.0 Mbp","3.5 Mbp","4.0 Mbp",
            "4.5 Mbp","5.0 Mbp","5.5 Mbp","6.0 Mbp","6.5 Mbp","7.0 Mbp","7.5 Mbp"))+
  scale_color_manual(values = c("#F8766D","#01BE67","#C77CFF"))+
  scale_fill_manual(values = c("#CD9600","#FF61CC","#00A9FF","#7CAE00","#00BFC4"))+
  coord_polar(start = pi / 23.5)+
  theme_classic()+
  theme(axis.text.y = element_blank(),
        axis.ticks.y = element_line(size = 0),
        axis.line = element_line(size = 0),
        axis.ticks.x = element_line(color = "black",size=1))






#################################################
## DECIPHER (Figure 3, Figure 4A-C, Figure S1) ##
#################################################

library(DECIPHER)
# For more information on DECIPHER, see http://www2.decipher.codes/.

#=======================
# sequence file reading
#=======================
# If you change the sequence file information below to genecluser information, you will see the results in figure 4A-C.
# The sequences of figure 4A-C were obtained through NCBI blast.
genomes <- c(`Amycolatopsis japonica MG417-CF17`="path/to/Amycolatopsis_japonica_MG417-CF17.fna",
             `Actinosynnema mirum DSM 43827`="path/to/Actinosynnema_mirum_DSM_43827.fna",
             `Catenulispora acidiphila DSM_44928`="path/to/Catenulispora_acidiphila_DSM_44928.fna",
             `S. rubrolavendulae MJM4426`="path/to/S._rubrolavendulae_MJM4426.fna",
             `S. venezuelae ATCC 10712`="path/to/S._venezuelae_ATCC_10712.fna",
             `S. formicae KY5`="path/to/S._formicae_KY5.fna",
             `S. alulus NK660`="path/to/S._alulus_NK660.fna",
             `S. pratensis ATCC 33331`="path/to/S._pratensis_ATCC_33331.fna",
             `S. hygroscopicus subsp. Jinggangensis 5008`="path/to/S._hygroscopicus_subsp._jinggangensis_5008.fna",
             `S. lincolnensis NRRL 2936`="path/to/S._lincolnensis_NRRL_2936.fna",
             `S. scabiei 87.22`="path/to/S._scabiei_87.22.fna",
             `S. avermitlis MA-4680`="path/to/S._avermitilis_MA-4680.fna",
             `S. coelicaolor A3(2)`="path/to/S._coelicolor.fna",
             `S. ambofaciens ATCC 23877`="path/to/S._ambofaciens_ATCC_23877.fna",
             `S. parvulus 2297`="path/to/S._parvulus_2297.fna",
             `S. cyaneogriseus NMWT 1`="path/to/S._cyaneogriseus_NMWT_1.fna",
             `S. malaysiensis DSM4137`="path/to/S._malaysiensis_DSM4137.fna",
             `S. violaceusniger Tu 4113`="path/to/S._violaceusniger_Tu_4113.fna",
             `S. globisporus S4-7`="path/to/S4-7.fna",
             `S. globisporus TFH56`="path/to/TFH56.fasta",
             `S. globisporus C-1027`="path/to/S._globisporus_C-1027.fna",
             `S. griseus NBRC 13350`="path/to/S._griseus_NBRC_13350.fna",
             `Kitasatospora albologa YIM 101047`="path/to/Kitasatospora_albologa_YIM_101047.fna",
             `S. fulvissimus DSM 40593`="path/to/S._fulvissimus_DSM_40593.fna"
             )

dbConn <- dbConnect(SQLite(), ":memory:")
for (i in seq_along(genomes)) {
  Seqs2DB(genomes[i],
          "FASTA",
          dbConn,
          names(genomes)[i],
          verbose=FALSE)
}
#========================
# find syntenic blocks
#=======================
synteny <- FindSynteny(dbConn)

#========================
# display a dot plot
#========================
labels <- sapply(lapply(strsplit(rownames(synteny), " "),
                        abbreviate,
                        minlength = 9),
                 paste,
                 collapse="\n")

pairs(synteny, labels=labels) 




############################
## Figure 2 and Figure S1 ##
############################

Heatmap with pyani results (Figure 3)
library(gplots)
library(RColorBrewer)

#==============================
# Input pyani identity result 
#==============================
# Installation and tutorial of pyani can be found below.
# https://github.com/widdowquinn/pyani
# In this study the result was obtained by typing 'average_nucleotide_identity.py -i path/to/24 fasta files folder/ -o /ANI_base_on_mummer_output -m ANIm' in bsh on ubuntu 18.04.
# -i : input folder
# -o : output results folder
# -m : method, ANIm means ��ANI base on mummer��

identity <- read.csv("path/to/ANIm_percentage_identity.tab", sep="\t", row.names = 1)
identity_percent<-as.matrix(round(identity,4)*100)

#========================
# Cluster rows and cols
#========================
hc.rows <- hclust(dist(identity_percent))
hc.cols <- hclust(dist(t(identity_percent)))

#=================
# Color setting
#=================
cols <- colorRampPalette(c("gray80","gray85", "gray99", "yellow", "red"))(50)

#================
# Draw heatmap (Figure 3)
#================
# The completed heatmap was edited in a staircase and merged with the DICIPHER results.
heatmap.2(identity_percent, trace="none",
          Colv=rev(as.dendrogram(hc.cols)), 
          Rowv=as.dendrogram(hc.rows), 
          scale='none',
          density.info="none",
          col=cols,
          cellnote=round(identity_percent,1), 
          notecol="black",
          rowsep=1:23,
          colsep = 1:23,
          margins=c(12,19),
          key.par=list(mar=c(1,1,1,1)),
          keysize=0.5
          )

AntiSMASH visualization (Figure 4C-D)
library(ggplot2)
#==================
# Input CDS information
#=================
# Based on the results of AntiSMASH, organism, CDS start, CDS end, and Legend were collected and saved as cvs file like to gff format.
# For easy understanding,
# + strand, the cluster start point was transformed to 0 bp.
gene_cluster_information <- read.csv("path/to/gene cluster information.csv")

#organism reorder
gene_cluster_information$organism <-  factor(gene_cluster_information$organism, 
                                             levels = c("S.globisporus C-1027",
                                                        "S.globisporus TFH56",
                                                        "S.globisporus S4-7",
                                                        "S.griseus NBRC13350",
                                                        "K.albolonga YIM 101047",
                                                        "S.fulvissimus DSM 40593"
                                                        )
)
# Data transform to numberic information
gene_cluster_information$strand_num <- ifelse(gene_cluster_information$strand == "+", 1 , 0)
gene_cluster_information$organism_order_num<- as.numeric(gene_cluster_information$organism)
     
#===============
# Color preset
#===============
gene_cluster_information$Legend <- factor(gene_cluster_information$Legend, levels = c("Additiional biosyntetic gene",
"Core biosynthetic gene",
"Regulatory gene",
"Transport-related gene",
"Other gene")
)
Color_preset <- c("#7CAB00", "#F77567", "#0EBECE", "#C57BFE", "grey80")

#===============
# Drawing figure
#===============
ggplot(data=gene_cluster_information)+
  geom_rect(mapping=aes(xmin=start,
                        xmax=end,
                        ymin=organism_order_num,
                        ymax=organism_order_num-0.3+0.6*strand_num, 
                        fill=Legend
                        ),size=0)+
  geom_hline(mapping=aes(yintercept=organism_order_num))+
  xlab("Length (bp)")+
  ylab("")+
  theme_classic()+
  theme(axis.line = element_blank(), axis.ticks.x = element_blank(),
        panel.grid.major.x = element_line(size = 0.1, colour = "grey"))+
  scale_y_continuous(breaks = 1:max(gene_cluster_information$organism_order_num),
                     labels=levels(gene_cluster_information$organism)[1:max(gene_cluster_information$organism_order_num)])+
  scale_fill_manual(values=color_preset)
